@@ -34,11 +34,10 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-groups.h>
-#include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-request-util.h>
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-contact-list-view.h"
@@ -397,7 +396,6 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 	const gchar   *contact_id = NULL;
 	gchar         *new_group = NULL;
 	gchar         *old_group = NULL;
-	gboolean       success = TRUE;
 	gboolean       new_group_is_fake, old_group_is_fake = TRUE;
 
 	priv = GET_PRIV (view);
@@ -441,7 +439,6 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 
 	if (!connection) {
 		DEBUG ("Failed to get connection for account '%s'", account_id);
-		success = FALSE;
 		g_free (new_group);
 		g_free (old_group);
 		g_object_unref (account_manager);
@@ -804,6 +801,17 @@ typedef struct {
 	guint32                 time;
 } MenuPopupData;
 
+static void
+menu_deactivate_cb (GtkMenuShell *menushell,
+		    gpointer user_data)
+{
+	/* FIXME: we shouldn't have to disconnec the signal (bgo #641327) */
+	g_signal_handlers_disconnect_by_func (menushell,
+		menu_deactivate_cb, user_data);
+
+	gtk_menu_detach (GTK_MENU (menushell));
+}
+
 static gboolean
 contact_list_view_popup_menu_idle_cb (gpointer user_data)
 {
@@ -822,8 +830,15 @@ contact_list_view_popup_menu_idle_cb (gpointer user_data)
 		gtk_menu_popup (GTK_MENU (menu),
 				NULL, NULL, NULL, NULL,
 				data->button, data->time);
-		g_object_ref_sink (menu);
-		g_object_unref (menu);
+
+		/* menu is initially unowned but gtk_menu_attach_to_widget () taked its
+		 * floating ref. We can either wait that the treeview releases its ref
+		 * when it will be destroyed (when leaving Empathy) or explicitely
+		 * detach the menu when it's not displayed any more.
+		 * We go for the latter as we don't want to keep useless menus in memory
+		 * during the whole lifetime of Empathy. */
+		g_signal_connect (menu, "deactivate", G_CALLBACK (menu_deactivate_cb),
+			NULL);
 	}
 
 	g_slice_free (MenuPopupData, data);
@@ -889,7 +904,7 @@ contact_list_view_row_activated (GtkTreeView       *view,
 
 	if (contact) {
 		DEBUG ("Starting a chat");
-		empathy_dispatcher_chat_with_contact (contact,
+		empathy_chat_with_contact (contact,
 			gtk_get_current_event_time ());
 		g_object_unref (contact);
 	}
@@ -921,7 +936,7 @@ contact_list_view_call_activated_cb (
 
 	event = (GdkEventButton *) gtk_get_current_event ();
 
-	menu = gtk_menu_new ();
+	menu = empathy_context_menu_new (GTK_WIDGET (view));
 	shell = GTK_MENU_SHELL (menu);
 
 	/* audio */
@@ -934,13 +949,9 @@ contact_list_view_call_activated_cb (
 	gtk_menu_shell_append (shell, item);
 	gtk_widget_show (item);
 
-	gtk_menu_attach_to_widget (GTK_MENU (menu),
-				   GTK_WIDGET (view), NULL);
 	gtk_widget_show (menu);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 			event->button, event->time);
-	g_object_ref_sink (menu);
-	g_object_unref (menu);
 
 	g_object_unref (contact);
 }
@@ -1873,7 +1884,6 @@ empathy_contact_list_view_new (EmpathyContactListStore        *store,
 EmpathyContact *
 empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 {
-	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
 	GtkTreeIter                iter;
 	GtkTreeModel              *model;
@@ -1881,8 +1891,6 @@ empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
-	priv = GET_PRIV (view);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return NULL;
@@ -1898,7 +1906,6 @@ empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 EmpathyContactListFlags
 empathy_contact_list_view_get_flags (EmpathyContactListView *view)
 {
-	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
 	GtkTreeIter                iter;
 	GtkTreeModel              *model;
@@ -1906,8 +1913,6 @@ empathy_contact_list_view_get_flags (EmpathyContactListView *view)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), 0);
 
-	priv = GET_PRIV (view);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return 0;
@@ -1924,7 +1929,6 @@ gchar *
 empathy_contact_list_view_get_selected_group (EmpathyContactListView *view,
 					      gboolean *is_fake_group)
 {
-	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
 	GtkTreeIter                iter;
 	GtkTreeModel              *model;
@@ -1934,8 +1938,6 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view,
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
-	priv = GET_PRIV (view);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return NULL;
