@@ -47,6 +47,7 @@
 #include "empathy-cell-renderer-activatable.h"
 #include "empathy-gtk-enum-types.h"
 #include "empathy-gtk-marshal.h"
+#include "empathy-ui-utils.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include <libempathy/empathy-debug.h>
@@ -236,29 +237,28 @@ cell_set_background (EmpathyPersonaView *self,
     GtkCellRenderer *cell,
     gboolean is_active)
 {
-  GdkColor  color;
-  GtkStyle *style;
-
-  style = gtk_widget_get_style (GTK_WIDGET (self));
-
   if (is_active)
     {
-      color = style->bg[GTK_STATE_SELECTED];
+      GdkRGBA color;
+      GtkStyleContext *style;
+
+      style = gtk_widget_get_style_context (GTK_WIDGET (self));
+
+      gtk_style_context_get_background_color (style, GTK_STATE_FLAG_SELECTED,
+          &color);
 
       /* Here we take the current theme colour and add it to
        * the colour for white and average the two. This
        * gives a colour which is inline with the theme but
        * slightly whiter.
        */
-      color.red = (color.red + (style->white).red) / 2;
-      color.green = (color.green + (style->white).green) / 2;
-      color.blue = (color.blue + (style->white).blue) / 2;
+      empathy_make_color_whiter (&color);
 
-      g_object_set (cell, "cell-background-gdk", &color, NULL);
+      g_object_set (cell, "cell-background-rgba", &color, NULL);
     }
   else
     {
-      g_object_set (cell, "cell-background-gdk", NULL, NULL);
+      g_object_set (cell, "cell-background-rgba", NULL, NULL);
     }
 }
 
@@ -354,14 +354,11 @@ individual_drag_received (EmpathyPersonaView *self,
     GdkDragContext *context,
     GtkSelectionData *selection)
 {
-  EmpathyPersonaViewPriv *priv;
   EmpathyIndividualManager *manager = NULL;
   FolksIndividual *individual;
   const gchar *individual_id;
   gboolean success = FALSE;
 
-  priv = GET_PRIV (self);
-
   individual_id = (const gchar *) gtk_selection_data_get_data (selection);
   manager = empathy_individual_manager_dup_singleton ();
   individual = empathy_individual_manager_lookup_member (manager,
@@ -410,11 +407,8 @@ drag_motion (GtkWidget *widget,
     guint time_)
 {
   EmpathyPersonaView *self = EMPATHY_PERSONA_VIEW (widget);
-  EmpathyPersonaViewPriv *priv;
   GdkAtom target;
 
-  priv = GET_PRIV (self);
-
   target = gtk_drag_dest_find_target (GTK_WIDGET (self), context, NULL);
 
   if (target == drag_atoms_dest[DND_DRAG_TYPE_INDIVIDUAL_ID])
@@ -452,15 +446,12 @@ drag_data_get (GtkWidget *widget,
     guint time_)
 {
   EmpathyPersonaView *self = EMPATHY_PERSONA_VIEW (widget);
-  EmpathyPersonaViewPriv *priv;
   FolksPersona *persona;
   const gchar *persona_uid;
 
   if (info != DND_DRAG_TYPE_PERSONA_ID)
     return;
 
-  priv = GET_PRIV (self);
-
   persona = empathy_persona_view_dup_selected (self);
   if (persona == NULL)
     return;
@@ -779,7 +770,6 @@ empathy_persona_view_new (EmpathyPersonaStore *store,
 FolksPersona *
 empathy_persona_view_dup_selected (EmpathyPersonaView *self)
 {
-  EmpathyPersonaViewPriv *priv;
   GtkTreeSelection *selection;
   GtkTreeIter iter;
   GtkTreeModel *model;
@@ -787,8 +777,6 @@ empathy_persona_view_dup_selected (EmpathyPersonaView *self)
 
   g_return_val_if_fail (EMPATHY_IS_PERSONA_VIEW (self), NULL);
 
-  priv = GET_PRIV (self);
-
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (self));
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     return NULL;
