@@ -32,6 +32,7 @@
 
 #include <libempathy-gtk/empathy-account-chooser.h>
 #include <libempathy-gtk/empathy-geometry.h>
+#include <libempathy-gtk/empathy-ui-utils.h>
 
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/interfaces.h>
@@ -901,7 +902,7 @@ debug_window_fill_service_chooser (EmpathyDebugWindow *debug_window)
   /* add Mission Control */
   gtk_list_store_append (priv->service_store, &iter);
   gtk_list_store_set (priv->service_store, &iter,
-      COL_NAME, "misson-control",
+      COL_NAME, "mission-control",
       COL_UNIQUE_NAME, "org.freedesktop.Telepathy.MissionControl5",
       -1);
 
@@ -1026,7 +1027,7 @@ debug_window_show_menu (gpointer user_data)
   GtkWidget *menu, *item;
   GtkMenuShell *shell;
 
-  menu = gtk_menu_new ();
+  menu = empathy_context_menu_new (GTK_WIDGET (data->debug_window));
   shell = GTK_MENU_SHELL (menu);
 
   item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
@@ -1039,8 +1040,6 @@ debug_window_show_menu (gpointer user_data)
 
   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
      data->button, data->time);
-  g_object_ref_sink (menu);
-  g_object_unref (menu);
 
   g_slice_free (MenuPopupData, user_data);
 
@@ -1408,6 +1407,8 @@ am_prepared_cb (GObject *am,
   gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), TRUE);
   gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar),
       GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_style_context_add_class (gtk_widget_get_style_context (toolbar),
+			       GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
   gtk_widget_show (toolbar);
 
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
